
import re
import logging
from odoo.tests import tagged
from odoo.addons.website.tests import test_crawl as website_test_crawl


_logger = logging.getLogger(__name__)


@tagged('post_install', '-at_install', 'crawl')
class Crawler(website_test_crawl.Crawler):
    # URL matching the following pattern will be discarded by the crawl
    # as they are targeting production URL or pages that don't exists in
    # local env.
    blacklisted_patterns = [
        '^/my/orders',   # external, should redirect to odoo website

        # apps referenced on the FAQ, but not avail locally (ex. on runbot)
        '^/apps/themes/13.0/theme_enark',
        '^/apps/themes/13.0/theme_treehouse/',
        '^/apps/themes/8.0/theme_clean/',
        '^/apps/modules/8.0/crm_voip/',
    ]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._blacklisted_patterns_used = set()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        # ensure blacklist patterns does not contains unused/deprecated patterns.
        unused_blacklisted_patterns = set(cls.blacklisted_patterns) - cls._blacklisted_patterns_used
        if unused_blacklisted_patterns:  # pragma: nocover
            _logger.warning(
                "No URL matched the following blacklisted pattern during any "
                "crawling test, please check if they are still relevant:\n%s",
                "\n".join(f"- {pattern}" for pattern in unused_blacklisted_patterns)
            )

    def crawl(self, url, seen=None, msg=''):
        if seen is None:
            seen = set()

        for pattern in self.blacklisted_patterns:
            if re.match(pattern, url):
                # skip fetching and crawling the page
                self._blacklisted_patterns_used.add(pattern)
                return seen

        seen = super().crawl(url, seen=seen, msg=msg)
        return seen
